@@ -23,7 +23,7 @@ class Channel(object):
     """
 
     def __init__(self, name):
-        self.name = name
+        self.name = name.lower()
         self.topic = None
         self.members = {}
         self.modes = {}
@@ -111,6 +111,7 @@ class Host(object):
     def remove_channel(self, channel):
         if isinstance(channel, Channel):
             channel = channel.name
+        channel = channel.lower()
         if channel not in self.channels:
             _log.warning("Ignoring request to remove a channel that hasn't "
                          "been added: '%s'", channel)
@@ -118,6 +119,20 @@ class Host(object):
         del self.channels[channel]
         _log.info("Left channel %s.", channel)
 
+    def get_channel(self, channel):
+        if isinstance(channel, Channel):
+            channel = channel.name
+        channel = channel.lower()
+        if channel not in self.channels:
+            _log.warning("Ignoring request to get a channel that hasn't "
+                         "been added: '%s'", channel)
+            return None
+        return self.channels[channel]
+
+    def in_channel(self, channel):
+        channel = str(channel).lower()
+        return channel in self.channels
+
 
 class Client(object):
     """An IRC client.
@@ -365,6 +380,10 @@ class Client(object):
         """Send a NOTICE to a user or channel."""
         self.send("NOTICE", target, ":" + message)
 
+    def topic(self, target, message):
+        """Sets TOPIC for a channel."""
+        self.send("TOPIC", target, ":" + message)
+
     def ctcp(self, target, message):
         """Send a CTCP message to a user or channel."""
         self.msg(target, "\x01%s\x01" % message)
@@ -386,7 +405,7 @@ class Client(object):
                          "with one of '%s': %s", chantypes, target)
             return False
 
-        if target in self.server.channels:
+        if self.server.in_channel(target):
             _log.warning("Ignoring request to join channel '%s' because we "
                          "are already in that channel.", target)
             return False
@@ -401,7 +420,7 @@ class Client(object):
 
     def part(self, target, message=None):
         """Part a channel."""
-        if str(target) not in self.server.channels:
+        if not self.server.in_channel(target):
             _log.warning("Ignoring request to part channel '%s' because we "
                          "are not in that channel.", target)
             return
@@ -437,7 +456,7 @@ class Client(object):
 
         (Values for modes which do not take arguments are ignored.)
         """
-        if str(channel) not in self.server.channels:
+        if not self.server.in_channel(channel):
             _log.warning("Ignoring request to set modes in channel '%s' "
                          "because we are not in that channel.", channel)
             return
@@ -614,7 +633,7 @@ def _parse_msg(client, command, actor, args):
     recipient, _, message = args.partition(' :')
     chantypes = client.server.features.get("CHANTYPES", "#")
     if recipient[0] in chantypes:
-        recipient = client.server.channels.get(recipient) or recipient
+        recipient = client.server.get_channel(recipient) or recipient.lower()
     else:
         recipient = User(recipient)
     client.dispatch_event(command, actor, recipient, message)
@@ -640,11 +659,11 @@ def _parse_join(client, command, actor, args):
         - MEMBERS, because the channel's members changed
     """
     actor = User(actor)
-    channel = args.lstrip(' :')
+    channel = args.lstrip(' :').lower()
     if actor.nick == client.user.nick:
         client.server.add_channel(channel)
         client.user.host = actor.host # now we know our host per the server
-    channel = client.server.channels[channel]
+    channel = client.server.get_channel(channel)
     channel.add_user(actor)
     client.dispatch_event("JOIN", actor, channel)
     if actor.nick != client.user.nick:
@@ -662,7 +681,7 @@ def _parse_part(client, command, actor, args):
     """
     actor = User(actor)
     channel, _, message = args.partition(' :')
-    channel = client.server.channels[channel]
+    channel = client.server.get_channel(channel)
     channel.remove_user(actor)
     if actor.nick == client.user.nick:
         client.server.remove_channel(channel)
@@ -699,7 +718,7 @@ def _parse_kick(client, command, actor, args):
     actor = User(actor)
     args, _, message = args.partition(' :')
     channel, target = args.split()
-    channel = client.server.channels[channel]
+    channel = client.server.get_channel(channel)
     channel.remove_user(target)
     target = User(target)
     if target.nick == client.user.nick:
@@ -712,7 +731,7 @@ def _parse_kick(client, command, actor, args):
 def _parse_topic(client, command, actor, args):
     """Parse a TOPIC and update channel state, then dispatch a TOPIC event."""
     channel, _, topic = args.partition(" :")
-    channel = client.server.channels[channel]
+    channel = client.server.get_channel(channel)
     channel.topic = topic or None
     if actor:
         actor = User(actor)
@@ -775,7 +794,7 @@ def _parse_namreply(client, command, actor, args):
     channelinfo, _, useritems = args.partition(' :')
     _, _, channel = channelinfo.rpartition(' ')  # channeltype channelname
 
-    c = client.server.channels.get(channel)
+    c = client.server.get_channel(channel)
     if not c:
         _log.warning("Ignoring NAMREPLY for channel '%s' which we are not in.",
             channel)
@@ -801,7 +820,7 @@ def _parse_endofnames(client, command, actor, args):
     """Parse an ENDOFNAMES and dispatch a NAMES event for the channel."""
     args = args.split(" :", 1)[0] # Strip off human-readable message
     _, _, channel = args.rpartition(' ')
-    channel = client.server.channels.get(channel) or channel
+    channel = client.server.get_channel(channel) or channel.lower()
     client.dispatch_event('MEMBERS', channel)
 
 
@@ -825,7 +844,7 @@ def _parse_mode(client, command, actor, args):
         return
 
     # channel-specific modes
-    chan = client.server.channels[channel]
+    chan = client.server.get_channel(channel)
 
     user_modes = set(client._get_prefixes().itervalues())
 
@@ -844,7 +863,7 @@ def _parse_mode(client, command, actor, args):
                 argument, tokens = tokens[0], tokens[1:]
 
             if mode in user_modes:
-                user = client.server.channels[channel].members[argument]
+                user = client.server.get_channel(channel).members[argument]
                 if op == "+":
                     user.modes.add(mode)
                 else:
@@ -955,6 +974,17 @@ def _parse_nick(client, command, actor, args):
 def _parse_invite(client, command, actor, args):
     """Parse an INVITE and dispatch an event."""
     target, _, channel = args.rpartition(" ")
-    client.dispatch_event("INVITE", actor, target, channel)
+    client.dispatch_event("INVITE", actor, target, channel.lower())
+    
+    
+@parser("NICKNAMEINUSE")
+def _parse_nicknameinuse(client, command, actor, args):
+    """Parse a NICKNAMEINUSE message and dispatch an event.
+    
+    The parameter passed along with the event is the nickname
+    which is already in use.
+    """
+    nick, _, _ = args.rpartition(" ")
+    client.dispatch_event("NICKNAMEINUSE", nick)
 
 # vim: set ts=4 sts=4 sw=4 et:
